#include "application.hpp"
#include "map.hpp"

Application::Application() : m_paused(false) {}

Application::~Application()
{
    for(unsigned int i(0); i < m_States.size(); i++)
        delete m_States[i];
}

void Application::start()
{
    sf::Event event;

    while(m_Window.isOpen())
    {
        while(m_Window.pollEvent(event))
        {
            if(event.type == sf::Event::MouseMoved)
            {
                m_RManager.setMousePosition(sf::Vector2i(event.mouseMove.x, event.mouseMove.y));
            }
            else if(event.type == sf::Event::Closed)
                m_Window.close();
            else if(event.type == sf::Event::KeyReleased)
            {
                if(event.key.code == sf::Keyboard::F1)
                    m_showFPS = !m_showFPS;

                else if(event.key.code == sf::Keyboard::F12)
                {
                    mainMenu();
                    popGameState();
                }
                else if(event.key.code == sf::Keyboard::F11)
                    mainMenu();
            }

            eventGS(event);
        }

        updateGS();

        m_tempsEntreImages.restart();

        m_Window.clear();

        drawGS();
        m_RManager.drawCursor();

        if(m_showFPS)
            this->ShowFPSCounter();

        m_Window.display();
    }
}

void Application::drawGS()
{
    for(int i(m_States.size() - 1); i >= 0; i--)
    {
        if(m_States[i]->isOpaque() || i == 0)
        {
            while(i < m_States.size())
            {
                m_States[i]->draw();
                i++;
            }

            return;
        }
    }
}

void Application::eventGS(sf::Event& event)
{
    int i(m_States.size() - 1);

    while(i >= 0)
    {
        if(m_States[i]->event(event))
            return;

        if(m_States[i]->isModal())
            return;

        i--;
    }
}

void Application::updateGS()
{
    if(!m_paused)
    {
        for(int i(m_States.size() - 1); i >= 0; i++)
        {
            m_States[i]->update(m_tempsEntreImages.getElapsedTime().asSeconds());

            if(m_States[i]->isModal())
                return;
        }
    }
}

void Application::addGameState(GameState* gs)
{
    m_States.push_back(gs);
}

void Application::popGameState()
{
    if(m_States.size() > 0)
    {
        delete m_States.back();
        m_States.pop_back();
    }

    if(m_States.size() == 0)
        m_Window.close();
}

void Application::init()
{
    m_RManager.setApplication(this);
    m_Window.create(sf::VideoMode(1000, 500, 32), "The F236 Bit - v." + VERSION, sf::Style::Default);
    m_Window.setVerticalSyncEnabled(false);
    m_RManager.setWindow(&m_Window);
    m_Window.setMouseCursorVisible(false);

    addGameState(new MainMenu(&m_RManager));

    m_showFPS = true;
    m_fps = 0;
    m_compteurFPS.setFont(m_RManager.getFont("data\\fonts\\f236.ttf"));
    m_compteurFPS.setCharacterSize(25);
    m_compteurFPS.setPosition(875, 0);

    m_Window.setIcon(170, 170, m_RManager.getTexture("data\\graphics\\window\\iconeFenetre.png").copyToImage().getPixelsPtr());
}

void Application::mainMenu()
{
    while(m_States.size() > 1)
    {
        popGameState();
    }
}

void Application::ShowFPSCounter()
{
    if(m_tempsFPS.getElapsedTime().asSeconds() >= 1.f)
    {
        std::ostringstream oss;
        oss << "FPS : " << m_fps;
        m_compteurFPS.setString(oss.str());
        m_tempsFPS.restart();
        m_fps = 0;
    }else
        m_fps++;

    m_Window.draw(m_compteurFPS);
}

void Application::pauseTime()
{
    m_paused = true;
}

void Application::resumeTime()
{
    m_paused = false;
    m_tempsEntreImages.restart();
}
