#ifndef APPLICATION_HPP
#define APPLICATION_HPP

/**
 * \file application.hpp
 * \brief Classe qui contient, intialise et lance la boucle de jeu
 * \author GIAFFERI Bastien
 * \version 1.0
 * \date 02 Mai 2013
 *
 * Classe centrale de l'application
 *
 */

// C++
#include <vector>
#include <sstream>
// SFML
// Projet
#include "constantes.hpp"
#include "gamestate.hpp"
#include "mainmenu.hpp"

class Application
{
public:
    Application();
    ~Application();

    /**
     * \fn void init()
     * \brief Initialise l'application. Elle charge diverses ressources, et initialise le gestionnaire de ressources.
     *
     *
     * \return void
     */
    void init();
    void start();
    void drawGS();
    void updateGS();


    void eventGS(sf::Event&);
    /**
     * \fn void addGameState(GameState* gs)
     * \brief Ajoute un GameState  la pile.
     *
     * \param gs Le GameState  ajouter.
     * \return void
     */

    void addGameState(GameState*);
    void popGameState();
    void mainMenu();
    void ShowFPSCounter();
    void pauseTime();
    void resumeTime();

private:
    // C++
    bool m_showFPS, m_paused;
    int m_fps;
    // SFML
    sf::RenderWindow m_Window;
    sf::Clock m_tempsEntreImages, m_tempsFPS;
    sf::Text m_compteurFPS;
    // Projet
    RessourceManager m_RManager;
    std::vector<GameState*> m_States;
};

#endif // APPLICATION_HPP
