#include "consoleminigame.hpp"
#include "application.hpp"
#include "random.hpp"
#include "map.hpp"
#include "entity.hpp"

ConsoleMinigame::ConsoleMinigame(RessourceManager* rm) : GameState(rm, true, false),
    delaiTiret(0.5), tiretAffiche(true), tailleLigneMax(65), nbLigneMax(21),
    m_termine(false), m_peuxRentrerNb(true), m_peuxRentrerChar(true)
{
    m_background = new Sprite(m_RManager->getTexture("data\\graphics\\window\\console\\background.png"));
    m_background->setPosition(1000 / 2 - m_background->getTextureRect().width / 2,
                              500 / 2 - m_background->getTextureRect().height / 2);
    m_texteConsole.setPosition(m_background->getPosition().x + 15, m_background->getPosition().y + 15);
    m_texteConsole.setFont(m_RManager->getFont("data\\fonts\\dialogue.ttf"));
    m_texteConsole.setCharacterSize(12);
    m_texteConsole.setColor(sf::Color(17, 200, 32));
}

ConsoleMinigame::~ConsoleMinigame()
{
    m_RManager->killTexture(m_background->getTexture());
}

void ConsoleMinigame::update(float t)
{
    if(delaiTiret - t < 0.f)
    {
        delaiTiret = 0.5;
        tiretAffiche = !tiretAffiche;

        std::string texte(m_texteConsole.getString());

        // On efface le dernier caractre (tiret ou espace)
        texte.erase(texte.size() - 1);

        if(tiretAffiche)
            texte += "_";
        else
            texte += " ";

        m_texteConsole.setString(texte);
    }
    else
        delaiTiret -= t;
}

void ConsoleMinigame::draw()
{
    m_RManager->getWindow().draw(*m_background);
    m_RManager->getWindow().draw(m_texteConsole);
}

void ConsoleMinigame::refreshConsole()
{
    std::string texte(m_texteConsole.getString());

    // On check qu'il n'y ai pas trop de lignes
    int nbLignes(1);
    unsigned found = texte.find_first_of('\n');

    while (found != std::string::npos)
    {
        nbLignes++;
        found=texte.find_first_of('\n', found + 1);
    }

    if(nbLignes > nbLigneMax)
    {
        for(unsigned int i(0); i < nbLignes - nbLigneMax; i++)
        {
            texte.erase(texte.begin(), texte.begin() + texte.find_first_of('\n') + 1);
        }
    }

    // Maintenant on check qu'il n'y ai pas trop de caractre.
    int nbCaract(texte.size() - texte.find_last_of('\n') - 1);

    if(nbCaract > tailleLigneMax)
    {
        texte.erase(texte.begin() + texte.find_last_of('\n') + tailleLigneMax, texte.begin() + texte.size() - 1);

        if(m_peuxRentrerNb && !m_peuxRentrerChar)
        {
            for(unsigned int i(nbCaract - tailleLigneMax); i > 0; i--)
                m_nombreEntres.erase(m_nombreEntres.begin() + m_nombreEntres.size() - 1);
        }
        else if(m_peuxRentrerChar && !m_peuxRentrerNb)
        {
            for(unsigned int i(nbCaract - tailleLigneMax); i > 0; i--)
                m_charEntres.erase(m_charEntres.begin() + m_charEntres.size() - 1);
        }
    }

    m_texteConsole.setString(texte);
}

bool ConsoleMinigame::event(const sf::Event& Event)
{
    if(Event.type == sf::Event::KeyReleased && !m_termine)
    {
        std::string texte(m_texteConsole.getString());
        // On efface le dernier caractre (tiret ou espace)
        texte.erase(texte.size() - 1);

        if(Event.key.code == sf::Keyboard::Space && m_peuxRentrerChar)
            texte += " ";
        else if(Event.key.code == sf::Keyboard::Return)
        {
            while(m_nombreEntres.size() < tailleLigneMax - 4)
                m_nombreEntres.insert(m_nombreEntres.begin(), 0);

            if(check(texte))
            {
                texte += "\n\n DONE ! Press any key to continue ! ";
                m_termine = true;
                tailleLigneMax = 65;
            }
            else
                texte += "\n>: ";

            m_nombreEntres.clear();
            m_charEntres.clear();
        }
        else if(Event.key.code == sf::Keyboard::Delete || Event.key.code == sf::Keyboard::Back)
        {
            if(texte[texte.size() - 2] != ':')
            {
                texte.erase(texte.size() - 1);
                if(m_peuxRentrerNb && !m_peuxRentrerChar)
                    m_nombreEntres.erase(m_nombreEntres.end()-1);
                else if(!m_peuxRentrerNb && m_peuxRentrerChar)
                    m_charEntres.erase(m_charEntres.end()-1);
            }
        }
        if(m_peuxRentrerChar)
        {
            if(Event.key.code == sf::Keyboard::A)
            { texte += "a"; m_charEntres.push_back('a'); }
            else if(Event.key.code == sf::Keyboard::B)
            { texte += "b"; m_charEntres.push_back('b'); }
            else if(Event.key.code == sf::Keyboard::C)
            { texte += "c"; m_charEntres.push_back('c'); }
            else if(Event.key.code == sf::Keyboard::D)
            { texte += "d"; m_charEntres.push_back('d'); }
            else if(Event.key.code == sf::Keyboard::E)
            { texte += "e"; m_charEntres.push_back('e'); }
            else if(Event.key.code == sf::Keyboard::F)
            { texte += "f"; m_charEntres.push_back('f'); }
            else if(Event.key.code == sf::Keyboard::G)
            { texte += "g"; m_charEntres.push_back('g'); }
            else if(Event.key.code == sf::Keyboard::H)
            { texte += "h"; m_charEntres.push_back('h'); }
            else if(Event.key.code == sf::Keyboard::I)
            { texte += "i"; m_charEntres.push_back('i'); }
            else if(Event.key.code == sf::Keyboard::J)
            { texte += "j"; m_charEntres.push_back('j'); }
            else if(Event.key.code == sf::Keyboard::K)
            { texte += "k"; m_charEntres.push_back('k'); }
            else if(Event.key.code == sf::Keyboard::L)
            { texte += "l"; m_charEntres.push_back('l'); }
            else if(Event.key.code == sf::Keyboard::M)
            { texte += "m"; m_charEntres.push_back('m'); }
            else if(Event.key.code == sf::Keyboard::N)
            { texte += "n"; m_charEntres.push_back('n'); }
            else if(Event.key.code == sf::Keyboard::O)
            { texte += "o"; m_charEntres.push_back('o'); }
            else if(Event.key.code == sf::Keyboard::P)
            { texte += "p"; m_charEntres.push_back('p'); }
            else if(Event.key.code == sf::Keyboard::Q)
            { texte += "q"; m_charEntres.push_back('q'); }
            else if(Event.key.code == sf::Keyboard::R)
            { texte += "r"; m_charEntres.push_back('r'); }
            else if(Event.key.code == sf::Keyboard::S)
            { texte += "s"; m_charEntres.push_back('s'); }
            else if(Event.key.code == sf::Keyboard::T)
            { texte += "t"; m_charEntres.push_back('t'); }
            else if(Event.key.code == sf::Keyboard::U)
            { texte += "u"; m_charEntres.push_back('u'); }
            else if(Event.key.code == sf::Keyboard::V)
            { texte += "v"; m_charEntres.push_back('v'); }
            else if(Event.key.code == sf::Keyboard::W)
            { texte += "w"; m_charEntres.push_back('w'); }
            else if(Event.key.code == sf::Keyboard::X)
            { texte += "x"; m_charEntres.push_back('x'); }
            else if(Event.key.code == sf::Keyboard::Y)
            { texte += "y"; m_charEntres.push_back('y'); }
            else if(Event.key.code == sf::Keyboard::Z)
            { texte += "z"; m_charEntres.push_back('z'); }
        }
        if(m_peuxRentrerNb)
        {
            if(Event.key.code == sf::Keyboard::Num0 || Event.key.code == sf::Keyboard::Numpad0)
            { texte += "0"; m_nombreEntres.push_back(0); }
            else if(Event.key.code == sf::Keyboard::Num1 || Event.key.code == sf::Keyboard::Numpad1)
            { texte += "1"; m_nombreEntres.push_back(1); }
            else if(Event.key.code == sf::Keyboard::Num2 || Event.key.code == sf::Keyboard::Numpad2)
            { texte += "2"; m_nombreEntres.push_back(2); }
            else if(Event.key.code == sf::Keyboard::Num3 || Event.key.code == sf::Keyboard::Numpad3)
            { texte += "3"; m_nombreEntres.push_back(3); }
            else if(Event.key.code == sf::Keyboard::Num4 || Event.key.code == sf::Keyboard::Numpad4)
            { texte += "4"; m_nombreEntres.push_back(4); }
            else if(Event.key.code == sf::Keyboard::Num5 || Event.key.code == sf::Keyboard::Numpad5)
            { texte += "5"; m_nombreEntres.push_back(5); }
            else if(Event.key.code == sf::Keyboard::Num6 || Event.key.code == sf::Keyboard::Numpad6)
            { texte += "6"; m_nombreEntres.push_back(6); }
            else if(Event.key.code == sf::Keyboard::Num7 || Event.key.code == sf::Keyboard::Numpad7)
            { texte += "7"; m_nombreEntres.push_back(7); }
            else if(Event.key.code == sf::Keyboard::Num8 || Event.key.code == sf::Keyboard::Numpad8)
            { texte += "8"; m_nombreEntres.push_back(8); }
            else if(Event.key.code == sf::Keyboard::Num9 || Event.key.code == sf::Keyboard::Numpad9)
            { texte += "9"; m_nombreEntres.push_back(9); }
        }

        if(tiretAffiche)
            texte += "_";
        else
            texte += " ";

        m_texteConsole.setString(texte);

        refreshConsole();
    }
    else if(Event.type == sf::Event::KeyReleased && m_termine)
        m_RManager->getApplication().popGameState();

    return m_modal;
}

PlusOuMoins::PlusOuMoins(RessourceManager* rm, int nbCaractere) : ConsoleMinigame(rm), m_nbCaractere(nbCaractere), m_code(0)
{
    tailleLigneMax = m_nbCaractere + 4;

    m_peuxRentrerChar = false;

    m_texteConsole.setString("FREQUENCY #%i\n\n>:  ");

    for(unsigned int i(0); i < nbCaractere; i++)
    {
        m_code += random(0, 9) *  pow(10, i);
    }
}

PlusOuMoins::~PlusOuMoins() {}

bool PlusOuMoins::check(std::string& texte)
{
    double nbEntre(0);

    for(int i(0); i < m_nombreEntres.size(); i++)
    {
        nbEntre = nbEntre + m_nombreEntres[m_nombreEntres.size() - 1 - i] * pow(10, i);
    }

    if(nbEntre < m_code)
    {
        texte += "\n\n C'est plus !\n";
    }else if(nbEntre > m_code)
    {
        texte += "\n\n C'est moins !\n";
    }else
    {
        texte += "\n\n Parfait !\n";
        return true;
    }

    return false;
}

MasterMind::MasterMind(RessourceManager* rm, int nbCaractere, int idAActiver) : ConsoleMinigame(rm), m_nbCaractere(nbCaractere),
    m_idAActiver(idAActiver)
{
    tailleLigneMax = m_nbCaractere + 4;

    m_peuxRentrerChar = false;

    m_texteConsole.setString("CODE #[-----]\n \"-\" -> Le chiffre n'est pas dans le code\n \"=\" -> Le chiffre est dans le code mais pas  cette position\n \"*\" -> Le chiffre est dans le code et  cette position\n\n>:  ");

    for(unsigned int i(0); i < nbCaractere; i++)
    {
        m_code.push_back(random(0, 9));
    }

    for(unsigned int i (0); i < m_code.size(); i++)
    {
        std::cout << m_code[i];
    }
}

MasterMind::~MasterMind() {}

bool MasterMind::check(std::string& texte)
{
    texte += "\n\n [";
    bool correct(true);

    for(unsigned int i(0); i < m_nombreEntres.size(); i++)
    {
        switch(m_nombreEntres[i])
        {
        case 0:
            texte += "0";
            break;
        case 1:
            texte += "1";
            break;
        case 2:
            texte += "2";
            break;
        case 3:
            texte += "3";
            break;
        case 4:
            texte += "4";
            break;
        case 5:
            texte += "5";
            break;
        case 6:
            texte += "6";
            break;
        case 7:
            texte += "7";
            break;
        case 8:
            texte += "8";
            break;
        case 9:
            texte += "9";
            break;
        }
    }

    texte += "]  <-  CODE\n [";

    for(int i(0); i < m_nombreEntres.size(); i++)
    {
        if(m_nombreEntres[i] == m_code[i])
        {
            texte += "*";
        }else
        {
            correct = false;
            bool existeDansLeCode(false);

            for(unsigned int j(0); j < m_code.size(); j++)
            {
                if(m_nombreEntres[i] == m_code[j])
                    existeDansLeCode = true;
            }

            if(existeDansLeCode)
                texte += "=";
            else
                texte += "-";
        }
    }

    texte += "]  <-  RIGHT CODE PRINT\n";

    if(correct)
    {
        texte += "\n\n CORRECT CODE - SUCCESS !\n";
        m_RManager->getMap()->activerEntite(m_idAActiver);
        return true;
    }else
        return false;
}
