#include "dialogue.hpp"
#include "application.hpp"
#include "pausemenu.hpp"
#include "map.hpp"
#include "player.hpp"

Actor::Actor(RessourceManager* rm, const std::string nom, const std::string background, const sf::Color couleur) :
    m_RManager(rm), m_backgroundName(background)
{
    m_nom.setFont(m_RManager->getFont("data\\fonts\\f236.ttf"));
    m_nom.setCharacterSize(18);
    m_nom.setString(nom);
    m_nom.setColor(couleur);

    m_avatar.setTexture(m_RManager->getTexture("data\\graphics\\actor\\" + nom + ".png"));
    m_background.setTexture(m_RManager->getTexture("data\\graphics\\window\\dialogue\\" + background + ".png"));
}

Actor::~Actor()
{
    m_RManager->killTexture(m_avatar.getTexture());
    m_RManager->killTexture(m_background.getTexture());
}

void Actor::draw()
{
    m_RManager->getWindow().draw(m_background);
    m_RManager->getWindow().draw(m_avatar);
    m_RManager->getWindow().draw(m_nom);
}

void Actor::setPosition(int x, int y)
{
    // (x, y) : point en haut  gauche du dialogue

    if(m_backgroundName == "backgroundFlecheG")
        m_background.setPosition(x - 23, y);
    else
        m_background.setPosition(x, y);

    m_avatar.setPosition(x + 1, y + 1);
    m_nom.setPosition(x + 106, y + 1);
}

const sf::Sprite& Actor::getBackground()
{ return m_background; }

Dialogue::Dialogue(RessourceManager* rm, const std::string path, int idAActiver) : GameState(rm, true, false),
    m_acteurActif(-1), m_idEntiterAActiver(idAActiver)
{
    m_RManager->getMap()->getPlayer()->resetMovement();

    m_dialogue.setFont(m_RManager->getFont("data\\fonts\\dialogue.ttf"));
    m_dialogue.setCharacterSize(12);

    m_waitForInput = new AnimatedSprite(m_RManager->getTexture("data\\graphics\\window\\dialogue\\wait.png"), 3, 1, 0.1);

    fichier.open(path.c_str());

    if(!fichier.is_open())
    {
        std::cout << " *** Echec de l'ouverture du fichier :\n    * " << path << std::endl;
        m_RManager->getApplication().popGameState();
        return;
    }

    this->load();
}

Dialogue::~Dialogue()
{
    for(unsigned int i(0); i < m_Actors.size(); i++)
        delete m_Actors[i];

    fichier.close();
}

void Dialogue::draw()
{
    if(m_acteurActif != -1)
    {
        m_Actors[m_acteurActif]->draw();
        m_RManager->getWindow().draw(m_dialogue);
        m_RManager->getWindow().draw(*m_waitForInput);
    }
}

void Dialogue::update(float t)
{
    m_waitForInput->update(t);
}

bool Dialogue::event(const sf::Event& evt)
{
    if(evt.type == sf::Event::KeyReleased)
    {
        if(evt.key.code == sf::Keyboard::Escape)
            m_RManager->getApplication().addGameState(new PauseMenu(m_RManager));
        else if((evt.key.code == sf::Keyboard::Return ||
                evt.key.code == sf::Keyboard::E || evt.key.code == sf::Keyboard::F || evt.key.code == sf::Keyboard::Space) && m_acteurActif != -1)
            chargerDialogue();
    }else if(evt.type == sf::Event::KeyReleased && m_acteurActif != -1)
    {
        if(evt.key.code == sf::Keyboard::Space)
            chargerDialogue();
    }else if(evt.type == sf::Event::MouseButtonReleased && m_acteurActif != -1)
    {
        if(collisionSourisSprite(evt.mouseButton.x, evt.mouseButton.y, m_Actors[m_acteurActif]->getBackground()))
            chargerDialogue();
    }

    return m_modal;
}

void Dialogue::load()
{
    std::string op("[Actor]");

    while (!fichier.eof() && op != "[Dialogue]")
    {
        fichier >> op;

        if(op == "[Actor]")
        {
            int r, g, b;
            std::string nom, background;
            fichier >> nom >> background >> r >> g >> b;
            m_Actors.push_back(new Actor(m_RManager, nom, background, sf::Color(r, g, b)));
        }
    }

    chargerDialogue();
}

void Dialogue::chargerDialogue()
{
    fichier.ignore();

    if(!fichier.eof())
    {
        int x, y, r, g, b;
        std::string dialogue;

        fichier >> m_acteurActif >> x >> y >> r >> g >> b;
        getline(fichier, dialogue, '#');

        if(x == -1)
            x = m_RManager->getWindow().getSize().x / 2 - m_Actors[m_acteurActif]->getBackground().getTextureRect().width / 2;
        if(y == -1)
            y = m_RManager->getWindow().getSize().y / 2 - m_Actors[m_acteurActif]->getBackground().getTextureRect().height / 2;

        m_Actors[m_acteurActif]->setPosition(x, y);
        m_dialogue.setString(dialogue);
        m_dialogue.setColor(sf::Color(r, g, b));
        m_dialogue.setPosition(x + 120, y + 8);

        m_waitForInput->setPosition(x + 579, y + 82);

        fichier.ignore();
    }else
    {
        if(m_idEntiterAActiver != -1)
            m_RManager->getMap()->activerEntite(m_idEntiterAActiver);

        m_RManager->getApplication().popGameState();
    }
}
