#include "entity.hpp"

Entity::Entity(RessourceManager* rm, Sprite* sprite, HitboxType hbType, int x, int y,
               bool solid, bool deadly, bool shootable, bool activable, float delay)
    : m_RManager(rm), m_sprite(sprite), m_typeHitbox(hbType),
      m_solid(solid), m_deadly(deadly), m_shootable(shootable), m_damageAmount(9999), m_activable(activable), m_delay(delay),
      m_tpsDepuisActivation(0.f), m_active(false), m_visible(true)
{
    if(m_sprite != 0)
    {
        m_sprite->setPosition(x, y);
        refreshHitbox();
    }
}

Entity::~Entity()
{
    if(m_sprite != 0)
        m_RManager->killTexture(m_sprite->getTexture());
}

void Entity::draw()
{
    if(m_sprite != 0 && m_visible)
        m_RManager->getWindow().draw(*m_sprite);
}

void Entity::update(float t)
{
    if(m_active && m_delay != -1)
    {
        m_tpsDepuisActivation += t;

        if(m_tpsDepuisActivation >= m_delay)
        {
            m_active = false;
            m_tpsDepuisActivation = 0.f;
        }
    }

    if(m_sprite != 0)
        if(m_sprite->update(t))
            refreshHitbox();
}

void Entity::activate() {}

void Entity::onShoot() {}

void Entity::onCollision() {}

void Entity::refreshHitbox()
{
     if(m_sprite != 0)
     {
         switch (m_typeHitbox)
         {
         case SpriteFilled:
             m_hitbox = sf::FloatRect(m_sprite->getPosition(),
                                      sf::Vector2f(m_sprite->getTextureRect().width, m_sprite->getTextureRect().height));
             break;

         default:
             m_hitbox = sf::FloatRect(0, 0, 0, 0);
             break;
         }
     }else
     {
         m_hitbox = sf::FloatRect(0, 0, 0, 0);
     }
}

bool Entity::isSolid() { return m_solid; }

bool Entity::isDeadly() { return m_deadly; }

int Entity::getDamage() { return m_damageAmount; }

bool Entity::isShootable() { return m_shootable; }

bool Entity::isActivable() { return m_activable; }

sf::FloatRect Entity::getHitbox() { return m_hitbox; }

const Sprite& Entity::getSprite() { return *m_sprite; }

void Entity::setVisible(bool visible)
{
    m_visible = visible;
}
