#include "loadingmenu.hpp"
#include "application.hpp"
#include "player.hpp"
#include "map.hpp"
#include "collisions.hpp"

LoadingMenu::LoadingMenu(RessourceManager* rm) : GameState(rm, true, true)
{
    // On charge les images
    m_background.setTexture(m_RManager->getTexture("data\\graphics\\window\\loadingMenu\\background.png"));
    m_flecheBack.setTexture(m_RManager->getTexture("data\\graphics\\window\\flecheBack.png"));
    m_flecheBack.setPosition(5, 5);

    // On regarde quelle sauvegarde existe
    std::ifstream fichier("data\\saves\\01\\save.sv");

    if(fichier.fail())
    {
        std::cout << " -*- Loading Menu : Slot 1 non present" << std::endl;
        m_slot_existe[0] = false;
    }
    else
    {
        std::cout << " -- Loading Menu : Slot 1 present" << std::endl;
        m_slot_existe[0] = true;

        std::string op;

        while(!fichier.eof())
        {
            fichier >> op;

            if(op == "[Map]")
            {
                std::string mapName;
                fichier >> mapName;
                m_slot_icon[0].setTexture(m_RManager->getTexture("data\\graphics\\map\\" + mapName + "\\mapIcon.png"));
                m_slot_icon[0].setPosition(348, 212);
            }
        }
    }

    fichier.close();
    fichier.open("data\\saves\\02\\save.sv");

    if(fichier.fail())
    {
        std::cout << " -*- Loading Menu : Slot 2 non present" << std::endl;
        m_slot_existe[1] = false;
    }
    else
    {
        std::cout << " -- Loading Menu : Slot 2 present" << std::endl;
        m_slot_existe[1] = true;

        std::string op;

        while(!fichier.eof())
        {
            fichier >> op;

            if(op == "[Map]")
            {
                std::string mapName;
                fichier >> mapName;
                m_slot_icon[1].setTexture(m_RManager->getTexture("data\\graphics\\map\\" + mapName + "\\mapIcon.png"));
                m_slot_icon[1].setPosition(348, 303);
            }
        }
    }

    fichier.close();
    fichier.open("data\\saves\\03\\save.sv");

    if(fichier.fail())
    {
        std::cout << " -*- Loading Menu : Slot 3 non present" << std::endl;
        m_slot_existe[2] = false;
    }
    else
    {
        std::cout << " -- Loading Menu : Slot 3 present" << std::endl;
        m_slot_existe[2] = true;

        std::string op;

        while(!fichier.eof())
        {
            fichier >> op;

            if(op == "[Map]")
            {
                std::string mapName;
                fichier >> mapName;
                m_slot_icon[2].setTexture(m_RManager->getTexture("data\\graphics\\map\\" + mapName + "\\mapIcon.png"));
                m_slot_icon[2].setPosition(348, 396);
            }
        }
    }

    fichier.close();
}

LoadingMenu::~LoadingMenu()
{
    for(unsigned int i(0); i < 3; i++)
        if(m_slot_existe[i])
            m_RManager->killTexture(m_slot_icon[i].getTexture());

    m_RManager->killTexture(m_background.getTexture());
    m_RManager->killTexture(m_flecheBack.getTexture());
}

void LoadingMenu::draw()
{
    m_RManager->getWindow().draw(m_background);

    for(unsigned int i(0); i < 3; i++)
        if(m_slot_existe[i])
            m_RManager->getWindow().draw(m_slot_icon[i]);

    m_RManager->getWindow().draw(m_flecheBack);
}

bool LoadingMenu::event(const sf::Event& e)
{
    if(e.type == sf::Event::MouseButtonReleased)
    {
        for(unsigned int i(0); i < 3; i++)
        {
            if(m_slot_existe[i])
            {
                if(collisionSourisSprite(e.mouseButton.x, e.mouseButton.y, m_slot_icon[i]))
                {
                    loadSlot(i);
                }
            }
        }

        if(collisionSourisSprite(e.mouseButton.x, e.mouseButton.y, m_flecheBack))
            m_RManager->getApplication().popGameState();
    }
    else if(e.type == sf::Event::KeyReleased)
    {
        if(e.key.code == sf::Keyboard::Escape)
        {
            m_RManager->getApplication().popGameState();
        }
    }
}

void LoadingMenu::loadSlot(unsigned int slot)
{
    std::string slotStr;

    if(slot == 0)
        slotStr = "01";
    else if(slot == 1)
        slotStr = "02";
    else if(slot == 2)
        slotStr = "03";

    std::string path("data\\saves\\" + slotStr + "\\save.sv");
    std::ifstream fichier(path.c_str());

    bool playerCanShoot, canToggleGravity;
    float playerShootDelay;
    std::string mapName, op;

    while(!fichier.eof())
    {
        fichier >> op;

        if(op == "[Player]")
        {
            fichier >> playerCanShoot;
            fichier >> playerShootDelay;
            fichier >> canToggleGravity;
        }
        else if(op == "[Map]")
        {
            fichier >> mapName;
        }
    }

    Player* player = new Player(m_RManager);
    player->peuxTirer(playerCanShoot);
    player->setShootDelay(playerShootDelay);
    player->canToggleGravity(canToggleGravity);

    m_RManager->getApplication().addGameState(new Map(m_RManager, mapName, path, player));
}
