#include "mainmenu.hpp"
#include "application.hpp"
#include "collisions.hpp"
#include "map.hpp"
#include "loadingmenu.hpp"
#include "newgamemenu.hpp"
#include "screen.hpp"

MainMenu::MainMenu(RessourceManager* RManager) : GameState(RManager, true, true), m_tpsAvantClignement(1.f)
{
    m_RManager->playBGM("mainMenu.ogg");
    m_playingBGM = true;

    m_background.setTexture(m_RManager->getTexture("data\\graphics\\window\\mainmenu\\background.png"));

    m_commencer.setTexture(m_RManager->getTexture("data\\graphics\\window\\mainmenu\\commencer.png"));
    m_commencer.setPosition(280, 213);

    m_charger.setTexture(m_RManager->getTexture("data\\graphics\\window\\mainmenu\\charger.png"));
    m_charger.setPosition(228, 289);

    m_quitter.setTexture(m_RManager->getTexture("data\\graphics\\window\\mainmenu\\quitter.png"));
    m_quitter.setPosition(387, 371);

    m_titre.setTexture(m_RManager->getTexture("data\\graphics\\window\\mainmenu\\titre.png"));
    m_titre.setPosition(59, 41);

    m_virus = new AnimatedSprite(m_RManager->getTexture("data\\graphics\\window\\mainmenu\\virus.png"), 4, 2, 0.1);
    m_virus->setPosition(449, 77);

    m_commandes.setTexture(m_RManager->getTexture("data\\graphics\\window\\commandes\\bouton.png"));
    m_commandes.setPosition(10, 450);
}

MainMenu::~MainMenu()
{
    m_RManager->stopBGM();

    delete m_virus;
}

bool MainMenu::event(const sf::Event& event)
{
    if(event.type == sf::Event::MouseButtonReleased)
    {
        if(collisionSourisSprite(event.mouseButton.x, event.mouseButton.y, m_commencer))
        {    
            m_RManager->getApplication().addGameState(new NewGameMenu(m_RManager)); // par dfaut : 01_USB
            m_playingBGM = false;
        }
        else if(collisionSourisSprite(event.mouseButton.x, event.mouseButton.y, m_charger))
        {
            m_RManager->getApplication().addGameState(new LoadingMenu(m_RManager));
            m_playingBGM = false;
        }
        else if(collisionSourisSprite(event.mouseButton.x, event.mouseButton.y, m_commandes))
        {
            m_RManager->getApplication().addGameState(new ScreenCommandes(m_RManager));
        }
        else if(collisionSourisSprite(event.mouseButton.x, event.mouseButton.y, m_quitter))
            m_RManager->getApplication().popGameState();
    }

    return true;
}

void MainMenu::draw()
{
    m_RManager->getWindow().draw(m_background);
    m_RManager->getWindow().draw(m_commencer);
    m_RManager->getWindow().draw(m_charger);
    m_RManager->getWindow().draw(m_quitter);
    m_RManager->getWindow().draw(*m_virus);
    m_RManager->getWindow().draw(m_titre);
    m_RManager->getWindow().draw(m_commandes);

    if(!m_playingBGM)
    {
        m_playingBGM = true;
        m_RManager->playBGM("mainMenu.ogg");
    }
}

void MainMenu::update(float t)
{
    if(m_virus->getEtatX() == 3 && m_virus->getEtatY() == 1)
    {
        m_tpsAvantClignement -= t;

        if(m_tpsAvantClignement <= 0)
        {
            m_tpsAvantClignement = 3.f;
            m_virus->setEtatX(0);
            m_virus->setEtatY(0);
        }
    }else
    {
        m_virus->update(t);
    }
}
