#ifndef MANAGER_HPP
#define MANAGER_HPP

///////////////////////////////////////////////////////////////
// Manager ////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////
// Date de cration     : 16/03/2013
// Dernire mise  jour : 16/03/2013
///////////////////////////////////////////////////////////////
/*
 * Cette classe de base gre les Ressources de type T.
 *
 * Elle associe chaque ID, de type string (en general le chemin
 * vers le fichier  charger)  la ressource elle-mme.
 * Elle conserve galement le nombre de fois ou la ressource a te
 * demande.
 * A chaque demande de suppression de la ressource, on dcrmente
 * le nombre de fois ou elle a t demande.
 * On ne supprime la ressource uniquement lorsque ce nombre arrive
 *  1
 * Prcision : On chargera la ressource  sa cration.
*/
///////////////////////////////////////////////////////////////

// C++
#include <iostream>
#include <string>
#include <vector>
// SFML
// Projet

template<typename T>
class Manager
{
public:
    Manager();
    ~Manager();

    const T& get(const std::string&);
    void kill(const std::string&);
    void kill(const T*);

protected:
    void virtual load() = 0;

    std::vector<T*> m_Item;
    std::vector<std::string> m_Path;
    std::vector<int> m_Count;
};

template<typename T>
Manager<T>::Manager() {}

template<typename T>
Manager<T>::~Manager()
{
    for(unsigned int i(0); i < m_Item.size(); i++)
    {
        delete m_Item[i];
        std::cout << " -*- ~Manager : Destruction de l'item\n    * " << m_Path[i] << std::endl;
    }

    m_Item.clear();
    m_Path.clear();
    m_Count.clear();
}

template<typename T>
const T& Manager<T>::get(const std::string& path)
{
    for(unsigned int id(0); id < m_Path.size(); id++)
    {
        if(m_Path[id] == path)
        {
            m_Count[id] ++;
            std::cout << " -- Manager : Demande de l'item : " << path << "\n    * Count = " << m_Count[id] << std::endl;
            return *m_Item[id];
        }
    }

    m_Path.push_back(path);
    m_Count.push_back(1);
    m_Item.push_back(new T);

    std::cout << " -- Manager : Creation de l'item : " << path << std::endl;

    load();

    return *m_Item.back();
}

template<typename T>
void Manager<T>::kill(const std::string& path)
{
    for(unsigned int i(0); i < m_Path.size(); i++)
    {
        if(m_Path[i] == path)
        {
            if(m_Count[i] == 1)
            {
                std::cout << " -*- Destruction de l'item : " << path << std::endl;
                m_Count.erase(m_Count.begin() + i);
                delete m_Item[i];
                m_Item.erase(m_Item.begin() + i);
                m_Path.erase(m_Path.begin() + i);
            }
            else
            {
                m_Count[i] --;
                std::cout << " -*- Tentative de destruction de l'item : " << path <<
                             "\n    * Count restant = " << m_Count[i] << std::endl;
            }

            return;
        }
    }
}

template<typename T>
void Manager<T>::kill(const T* item)
{
    for(unsigned int i(0); i < m_Item.size(); i++)
    {
        if(m_Item[i] == item)
            kill(m_Path[i]);
    }
}

#endif // MANAGER_HPP
