#include "map.hpp"
#include "application.hpp"
#include "pausemenu.hpp"
#include "consoleminigame.hpp"
#include "dialogue.hpp"

Map::Map(RessourceManager* rm, const std::string mapName, std::string pathSave, Player* player) :
    GameState(rm, true, true),
    m_player(player), m_background(0), m_pathSave(pathSave)
{
    m_RManager->setMap(this);

    if(player == 0)
        m_player = new Player(m_RManager);

    changeMap(mapName);
}

Map::~Map()
{
    save();

    if(m_background != 0)
        m_RManager->killTexture(m_background->getTexture());

    for(unsigned int i(0); i < m_Bullet.size(); i++)
        delete m_Bullet[i];

    for(std::map<int, Entity*>::iterator it = m_Entity.begin(); it != m_Entity.end(); it++)
        delete it->second;

    delete m_player;

    m_RManager->stopBGM();

    m_RManager->setMap(0);
}

void Map::draw()
{
    if(m_background != 0)
        m_RManager->getWindow().draw(*m_background);

    // On affiche les entits
    for(std::map<int, Entity*>::iterator it = m_Entity.begin(); it != m_Entity.end(); it++)
        it->second->draw();

    for(unsigned int i(0); i < m_Bullet.size(); i++)
        m_Bullet[i]->draw();

    // On affiche le joueur
    m_player->draw();
}

bool Map::event(const sf::Event& evt)
{
    if(evt.type == sf::Event::KeyReleased)
    {
        if(evt.key.code == sf::Keyboard::F5)
            changeMap(m_mapName);
        else if(evt.key.code == sf::Keyboard::F6)
            m_RManager->getApplication().addGameState(new PlusOuMoins(m_RManager, 3));
        else if(evt.key.code == sf::Keyboard::Escape)
            m_RManager->getApplication().addGameState(new PauseMenu(m_RManager));
        else if(evt.key.code == sf::Keyboard::F7)
            m_RManager->getApplication().addGameState(new Dialogue(m_RManager, "data\\maps\\test\\dialogue\\test.dia"));
    }else if(evt.type == sf::Event::KeyPressed)
    {
        if(evt.key.code == sf::Keyboard::E)
        {
            for(std::map<int, Entity*>::iterator it = m_Entity.begin(); it != m_Entity.end(); it++)
            {
                if(it->second->isActivable())
                {
                    if(m_player->getHitbox().intersects(it->second->getHitbox()))
                        it->second->activate();
                }
            }
        }
    }else if(evt.type == sf::Event::MouseButtonReleased && evt.mouseButton.button == sf::Mouse::Right)
    {
        for(std::map<int, Entity*>::iterator it = m_Entity.begin(); it != m_Entity.end(); it++)
        {
            if(it->second->isActivable())
            {
                if(m_player->getHitbox().intersects(it->second->getHitbox()) &&
                        collisionSourisSprite(evt.mouseButton.x, evt.mouseButton.y, it->second->getSprite()))
                    it->second->activate();
            }
        }
    }

    m_player->event(evt);

    return m_modal;
}

void Map::update(float t)
{
    // On update le background
    if(m_background != 0)
        m_background->update(t);

    // On update les entits
    for(std::map<int, Entity*>::iterator it = m_Entity.begin(); it != m_Entity.end(); it++)
        it->second->update(t);

    for(unsigned int i(0); i < m_Bullet.size(); i++)
        m_Bullet[i]->update(t);

    // On update le joueur
    m_player->update(t);

    // Check des collisions
    for(std::map<int, Entity*>::iterator it = m_Entity.begin(); it != m_Entity.end(); it++)
    {
        sf::FloatRect collision;

        if(m_player->getHitbox().intersects(it->second->getHitbox(), collision))
        {
            if(it->second->isSolid())
            {
                sf::FloatRect pHB = m_player->getHitbox();
                if(collision.width > collision.height)
                {
                    if(collision.top == pHB.top)
                    {
                        m_player->move(0, collision.height);
                        m_player->setAccelV(0.f);
                        if(!m_player->isGravityNormal())
                            m_player->setSol(it->second);
                    }
                    else
                    {
                        m_player->move(0, -collision.height);
                        if(m_player->isGravityNormal())
                            m_player->setSol(it->second);
                    }
                }
                else if(collision.width < collision.height)
                {
                    if(collision.left == pHB.left)
                        m_player->move(collision.width, 0);
                    else
                        m_player->move(-collision.width, 0);

                    m_player->setAccel(0.f);
                }

                if(it->second->isDeadly())
                {
                   m_player->takeDamage(it->second->getDamage());
                }

                it->second->onCollision();
            }
        }else if(m_player->getSol() == it->second)
        {
            if(it->second->isDeadly())
            {
                m_player->takeDamage(it->second->getDamage());
            }
        }

        if(it->second->isSolid() || it->second->isShootable())
        {
            for(unsigned int i(0); i < m_Bullet.size(); i++)
            {
                if(it->second->getHitbox().contains(m_Bullet[i]->getPosition()))
                {
                    if(it->second->isShootable())
                    {
                        it->second->onShoot();
                        delete m_Bullet[i];
                        m_Bullet.erase(m_Bullet.begin() + i);
                        i--;
                    }
                    else if(it->second->isSolid())
                    {
                        delete m_Bullet[i];
                        m_Bullet.erase(m_Bullet.begin() + i);
                        i--;
                    }
                }
            }
        }
    }
}

void Map::changeMap(const std::string mapName)
{
    m_RManager->getApplication().pauseTime();

    if(mapName != m_mapName)
        save();

    m_mapName = mapName;

    m_pathGraphics = "data\\graphics\\map\\" + mapName + "\\";

    if(m_background != 0)
    {
        m_RManager->killTexture(m_background->getTexture());
        delete m_background;
        m_background = 0;
    }

    // Suppression des entits actuelles
    for(unsigned int i(0); i < m_Bullet.size(); i++)
        delete m_Bullet[i];
    m_Bullet.clear();

    for(std::map<int, Entity*>::iterator it = m_Entity.begin(); it != m_Entity.end(); it++)
        delete it->second;
    m_Entity.clear();

    // On rinitialise le sol du player
    m_player->setSol(0);

    m_player->resetMovement();

    load("data\\maps\\" + mapName + "\\" + mapName + ".zi");

    m_RManager->getApplication().resumeTime();
}

Player* Map::getPlayer()
{
    return m_player;
}

std::string Map::getMapName()
{
    return m_mapName;
}

void Map::activerEntite(int id)
{
    if(m_Entity.find(id) != m_Entity.end())
        m_Entity.find(id)->second->activate();
    else
        std::cout << " -*- ERROR : Activation de l'entite impossible\n    * Entite : " << id << " n'existe pas" << std::endl;
}

void Map::addBullet(Bullet* bullet)
{
    m_Bullet.push_back(bullet);
}

void Map::save()
{
    std::ofstream fichier(m_pathSave.c_str());

    fichier << "[Map]\n" << m_mapName << "\n";

    m_player->save(fichier);
}

void Map::setEntityVisible(int id, bool visible)
{
    if(m_Entity.find(id) != m_Entity.end())
        m_Entity.find(id)->second->setVisible(visible);
}
