#include "map.hpp"
#include "trigger.hpp"
#include "structentity.hpp"
#include "mapentity.hpp"
#include "scriptedentity.hpp"

void Map::load(const std::string path)
{
    std::cout << " -- Chargement de la map : " << path << std::endl;

    std::ifstream fichier(path.c_str());
    std::string op;

    if(!fichier.is_open())
    {
        std::cout << " *** ERREUR : Echec de l'ouverture de la map :\n    * " << path << std::endl;
        return;
    }

    while(!fichier.eof())
    {
        fichier >> op;

        if(op == "|")
        {
            getline(fichier, op);
        }
        else if(op == "[Background]")
        {
            m_background = loadSprite(fichier);
        }
        else if(op == "[BGM]")
        {
            fichier >> op;
            m_RManager->playBGM(op);
        }
        else if(op == "[GroundEntity]")
        {
            int id, x, y;
            HitboxType hbType = None;

            fichier >> id >> x >> y;

            Sprite* sp = loadSprite(fichier);

            fichier >> op;

            if(op == "HitboxSpriteFilled")
            {
                hbType = SpriteFilled;
            }

            m_Entity.insert(std::make_pair(id,
                                           new GroundEntity(m_RManager, sp, hbType, x, y)));
        }
        else if(op == "[PicEntity]")
        {
            int id, x, y;

            fichier >> id >> x >> y;

            Sprite* sp = loadSprite(fichier);

            m_Entity.insert(std::make_pair(id,
                                           new PicEntity(m_RManager, sp, x, y)));
        }
        else if(op == "[ActivableTrigger]" || op == "[TimerActivableTrigger]")
        {
            std::string type(op);
            int id, x, y, idDest;
            float delay;

            fichier >> id >> x >> y >> idDest >> delay;

            Sprite* sp = loadSprite(fichier);
            Entity* dest = 0;

            if(m_Entity.find(idDest) != m_Entity.end())
                dest = m_Entity.find(idDest)->second;
            else
                std::cout << " *** ERROR : Trigger destination dont exist !\n    * destID : " << idDest << std::endl;

            if(type == "[ActivableTrigger]")
                m_Entity.insert(std::make_pair(id,
                                               new ActivableTrigger(m_RManager, sp, dest, x, y, delay)));
            else if(type == "[TimerActivableTrigger]")
                m_Entity.insert(std::make_pair(id,
                                               new TimerActivableTrigger(m_RManager, sp, dest, x, y, delay)));
        }
        else if(op == "[TimerShootableTrigger]")
        {
            int id, x, y, idDest;
            float delay;

            fichier >> id >> x >> y >> idDest >> delay;

            Sprite* sp = loadSprite(fichier);
            Entity* dest = 0;

            if(m_Entity.find(idDest) != m_Entity.end())
                dest = m_Entity.find(idDest)->second;
            else
                std::cout << " *** ERROR : Trigger destination dont exist !\n    * destID : " << idDest << std::endl;

            m_Entity.insert(std::make_pair(id,
                                           new TimerShootableTrigger(m_RManager, sp, dest, x, y, delay)));
        }
        else if(op == "[CurrentLevelVortex]")
        {
            int id, x, y;

            fichier >> id >> x >> y;

            m_Entity.insert(std::make_pair(id,
                                           new CurrentLevelVortex(m_RManager, x, y)));
        }
        else if(op == "[NextLevelVortex]")
        {
            int id, x, y;

            fichier >> id >> x >> y >> op;

            m_Entity.insert(std::make_pair(id,
                                           new NextLevelVortex(m_RManager, op, x, y)));
        }
        else if(op == "[ScriptedEntity]")
        {
            int id, x, y;

            fichier >> id >> x >> y >> op;

            Sprite* sp = loadSprite(fichier);

            m_Entity.insert(std::make_pair(id,
                                           new ScriptedEntity(m_RManager, sp, op, x, y)));
        }
        else if(op == "[DoorEntity]" || op == "[InversedDoorEntity]")
        {
            int id, x, y;

            fichier >> id >> x >> y;

            m_Entity.insert(std::make_pair(id,
                                           new DoorEntity(m_RManager, loadSprite(fichier), x, y)));

            if(op == "[InversedDoorEntity]")
                m_Entity.at(id)->activate();
        }
        else if(op == "[DecorationEntity]")
        {
            int id, x, y;

            fichier >> id >> x >> y;

            m_Entity.insert(std::make_pair(id,
                                           new DecorationEntity(m_RManager, loadSprite(fichier), x, y)));
        }
    }

    fichier.close();
}

Sprite* Map::loadSprite(std::ifstream& fichier)
{
    std::string spriteType, path;

    fichier >> spriteType;

    if(spriteType == "Sprite")
    {
        fichier >> path;
        return new Sprite(m_RManager->getTexture(m_pathGraphics + path));
    }
    else if(spriteType == "RepeatedSprite")
    {
        float vBas, vGauche;
        fichier >> path >> vBas >> vGauche;

        return new RepeatedSprite(m_RManager->getTexture(m_pathGraphics + path), vBas, vGauche);
    }
    else if(spriteType == "AnimatedSprite")
    {
        int nbEtatX, nbEtatY;
        float tpsEntreImages;
        bool changeEtatYToutSeul;

        fichier >> path >> nbEtatX >> nbEtatY >> tpsEntreImages >> changeEtatYToutSeul;

        return new AnimatedSprite(m_RManager->getTexture(m_pathGraphics + path), nbEtatX, nbEtatY, tpsEntreImages, changeEtatYToutSeul);
    }

    std::cout << " *** ERREUR : Chargement du sprite " << m_pathGraphics << path << std::endl;
}
