#include "player.hpp"
#include "application.hpp"
#include "screen.hpp"
#include "map.hpp"
#include "bullet.hpp"

Player::Player(RessourceManager* rm) : m_RManager(rm)
{
    init();
}

Player::~Player()
{
    m_RManager->killTexture(m_sprite->getTexture());
    m_RManager->killTexture(m_hud_Bulle.getTexture());
    m_RManager->killTexture(m_hud_barreVie.getTexture());
    m_RManager->killTexture(m_hud_vie.getTexture());

    m_RManager->deleteMouseCursor();

    delete m_sprite;
}

void Player::draw()
{
    m_RManager->getWindow().draw(*m_sprite);
    m_RManager->getWindow().draw(m_hud_barreVie);
    m_RManager->getWindow().draw(m_hud_vie);
    m_RManager->getWindow().draw(m_hud_Bulle);
}

void Player::update(float t)
{
    // On check si les touches sont appuyes
    if(sf::Keyboard::isKeyPressed(sf::Keyboard::Q) && !gauche)
    {
        gauche = true;
        m_sprite->setEtatY(0);
    }else if(!sf::Keyboard::isKeyPressed(sf::Keyboard::Q) && gauche)
    {
        gauche = false;
        m_sprite->setEtatX(0);
    }
    if(sf::Keyboard::isKeyPressed(sf::Keyboard::D) && !droite)
    {
        droite = true;
        m_sprite->setEtatY(1);
    }else if(!sf::Keyboard::isKeyPressed(sf::Keyboard::D) && droite)
    {
        droite = false;
        m_sprite->setEtatX(0);
    }

    if(sf::Keyboard::isKeyPressed(sf::Keyboard::Space) && m_sol != 0)
    {
        accelV = -accelMax;
        m_sol = 0;
    }

    // On calcule l'acclration
    calculerAcceleration(t);

    // On dplace le joueur
    if(gauche)
    {
        m_sprite->move(-t * vitesse * accel, 0.f);
    }

    if(droite)
    {
        m_sprite->move(t * vitesse * accel, 0.f);
    }

    if(haut && m_sol == 0)
    {
        m_sprite->move(0.f, -t * vitesse * accelV);
    }

    if(bas && m_sol == 0)
    {
        m_sprite->move(0.f, t * vitesse * accelV);
    }

    // Si le joueur  dpass les limites du sol, on lui retire le sol
    if(m_sol != 0 && (
                m_sprite->getPosition().x > m_sol->getHitbox().left + m_sol->getHitbox().width ||
                m_sprite->getPosition().x + m_sprite->getTextureRect().width < m_sol->getHitbox().left))
    {
        m_sol = 0;
        // Dans le doute, on remet l'acclration verticale  0
        accelV = 0.f;
    }

    // On regarde si il tire
    if(m_delaiJusqueProchainTir - t <= 0.f)
        m_delaiJusqueProchainTir = 0.f;
    else
        m_delaiJusqueProchainTir -= t;

    if(m_clicGaucheEnfonce && m_peuxTirer)
    {
        if(m_delaiJusqueProchainTir == 0.f)
        {
            m_RManager->getMap()->addBullet(new Bullet(m_RManager, 100, sf::FloatRect(m_sprite->getPosition(),
                                                                                      sf::Vector2f(m_RManager->getMousePosition().x - m_sprite->getPosition().x,
                                                                                                   m_RManager->getMousePosition().y - m_sprite->getPosition().y))));

            m_RManager->playSound("shoot");

            m_delaiJusqueProchainTir = m_intervalleTir;
        }
    }
}

void Player::event(const sf::Event& e)
{
    if(e.type == sf::Event::MouseButtonPressed && e.mouseButton.button == sf::Mouse::Left)
    {
        m_clicGaucheEnfonce = true;
    }else if(e.type == sf::Event::MouseButtonReleased && e.mouseButton.button == sf::Mouse::Left)
    {
        m_clicGaucheEnfonce = false;
    }else if(e.type == sf::Event::KeyReleased)
    {
        if(e.key.code == sf::Keyboard::LShift && m_peuxChangerGravite && m_sol != 0)
            toggleGravity();
    }
}

sf::FloatRect Player::getHitbox()
{
    return sf::FloatRect(m_sprite->getPosition(),
                       sf::Vector2f((float)m_sprite->getTextureRect().width, (float)m_sprite->getTextureRect().height));
}

void Player::setPosition(int x, int y)
{
    m_sprite->setPosition(x, y);
    droite = false;
    gauche = false;
    m_sol = 0;
}

void Player::resetMovement()
{
    droite = false;
    gauche = false;
    m_clicGaucheEnfonce = false;
    accel = 0.f;
    accelV = 0.f;
    m_sprite->setEtatX(0);
}

void Player::move(float x, float y)
{
    m_sprite->move(x, y);
}

void Player::setSol(Entity* sol)
{
    m_sol = sol;
    accelV = 0.f;
}

Entity* Player::getSol()
{ return m_sol; }

void Player::setAccelV(float aV)
{
    accelV = aV;
}

void Player::setAccel(float a)
{
    accel = a;
}

void Player::takeDamage(int amount)
{
    if(m_vie != 0)
    {
        m_vie -= amount;

        if(m_vie < 0)
        {
            m_vie = 0;
            m_RManager->getApplication().addGameState(new GameOver(m_RManager));
        }

        m_hud_vie.setTextureRect(sf::IntRect(0,
                                             0,
                                             m_hud_vie.getTexture()->getSize().x * ((float)(m_vie) / (float)(m_vieMax)),
                                             m_hud_vie.getTexture()->getSize().y));
    }
}

void Player::setVie(int vie)
{
    if(vie == -1)
        m_vie = m_vieMax;
    else
        m_vie == vie;

    m_hud_vie.setTextureRect(sf::IntRect(0,
                                         0,
                                         m_hud_vie.getTexture()->getSize().x * ((float)(m_vie) / (float)(m_vieMax)),
                                         m_hud_vie.getTexture()->getSize().y));
}

void Player::calculerAcceleration(float t)
{
    // Calcul de l'acclration du mouvement horizontal
    if(!gauche && !droite)
    {
        if(accel - pas * t > 0.f)
            accel -= pas * t;
        else if(accel != 0.f)
            accel = 0.f;
    }else
    {
        if(accel + pas * t < accelMax)
            accel += pas * t;
        else if(accel != accelMax)
            accel = accelMax;

        // On en profite pour rafraichir le sprite annim du joueur
        m_sprite->update(t);
    }

    // Calcul de l'acclration du mouvement vertical
    if(!haut && !bas)
    {
        if(accelV - pas * t > 0.f)
            accelV -= pas * t;
        else if(accelV != 0.f)
            accelV = 0.f;
    }else if (m_sol == 0)
    {
        if(accelV + pas * t < accelMax)
            accelV += pas * t;
        else if(accelV != accelMax)
            accelV = accelMax;
    }
}

void Player::peuxTirer(bool peux)
{
    if(peux && !m_peuxTirer)
    {
        m_peuxTirer = peux;
        m_RManager->setMouseCursor("viseur");
    }else if(!peux && m_peuxTirer)
        m_RManager->deleteMouseCursor();
}

void Player::setShootDelay(float delay)
{
    m_intervalleTir = delay;
}

void Player::init()
{
    m_sprite = new AnimatedSprite(m_RManager->getTexture("data\\graphics\\character\\v1.png"), 4, 2, 0.1, false);
    m_sprite->setPosition(10, 10);

    haut = false;
    bas = true;
    droite = false;
    gauche = false;

    vitesse = 300.f;
    accel = 0.f, accelV = 0.f;
    pas = 2.f;
    accelMax = 1.f;

    m_sol = 0;
    m_vieMax = 100;
    m_vie = 100;

    m_clicGaucheEnfonce = false;
    m_peuxTirer = false;
    m_degatArme = 15;
    m_intervalleTir = 0.5;
    m_delaiJusqueProchainTir = 0.f;

    m_hud_Bulle.setTexture(m_RManager->getTexture("data\\graphics\\window\\hud\\bulle.png"));
    m_hud_Bulle.setPosition(5, 5);
    m_hud_barreVie.setTexture(m_RManager->getTexture("data\\graphics\\window\\hud\\barreVie.png"));
    m_hud_barreVie.setPosition(48, 20);
    m_hud_vie.setTexture(m_RManager->getTexture("data\\graphics\\window\\hud\\vie.png"));
    m_hud_vie.setPosition(49, 21);

    m_peuxChangerGravite = false;
}

void Player::save(std::ofstream& fichier)
{
    fichier << "[Player]\n" << m_peuxTirer << " " << m_intervalleTir << " " << m_peuxChangerGravite;
}

void Player::toggleGravity()
{
    if(bas)
    {
        haut = true;
        bas = false;
        m_sprite->setOrigin(0.f, m_sprite->getTextureRect().height);
        m_sprite->setScale(-1.f, 1.f);
    }else
    {
        bas = true;
        haut = false;
        m_sprite->setOrigin(0.f, 0.f);
        m_sprite->setScale(1.f, 1.f);
    }

    m_sol = 0;
    m_sprite->rotate(180);
}

bool Player::isGravityNormal()
{
    return bas;
}

void Player::canToggleGravity(bool can)
{
    m_peuxChangerGravite = can;
}
