#include "ressourcemanager.hpp"
#include "application.hpp"
#include "map.hpp"
#include "random.hpp"

RessourceManager::RessourceManager() : m_Application(0), m_Map(0), m_pathBGM("")
{
    m_BGM.setLoop(true);
    m_BGM.setVolume(80);
    setMouseCursor("fleche");
}

RessourceManager::~RessourceManager()
{
    for(unsigned int i(0); i < m_Sound.size(); i++)
        delete m_Sound[i];

    for(unsigned int i(0); i < m_hud_curseur.size(); i++)
    {
        killTexture(m_hud_curseur.top()->getTexture());
        delete m_hud_curseur.top();
        m_hud_curseur.pop();
    }

    m_BGM.stop();
}

const sf::Texture& RessourceManager::getTexture(const std::string& path)
{ return m_Textures.get(path); }

void RessourceManager::killTexture(const std::string& path)
{ m_Textures.kill(path); }

void RessourceManager::killTexture(const sf::Texture* texture)
{ m_Textures.kill(texture); }

void RessourceManager::setApplication(Application* app)
{ m_Application = app; }

Application& RessourceManager::getApplication()
{ return *m_Application; }

void RessourceManager::setWindow(sf::RenderWindow* window)
{ m_Window = window; }

sf::RenderWindow& RessourceManager::getWindow()
{ return *m_Window; }

const sf::Font& RessourceManager::getFont(const std::string& path)
{ return m_Fonts.get(path); }

void RessourceManager::setMap(Map* map)
{ m_Map = map; }

Map* RessourceManager::getMap()
{ return m_Map; }

void RessourceManager::playSound(std::string path)
{
    if(path == "shoot")
    {
        path = "data\\sounds\\shoot";
        switch(random(0, 6))
        {
        case 0:
            path += "0";
            break;

        case 1:
            path += "1";
            break;

        case 2:
            path += "2";
            break;

        case 3:
            path += "3";
            break;

        case 4:
            path += "4";
            break;

        case 5:
            path += "5";
            break;

        case 6:
            path += "6";
            break;

        case 7:
            path += "7";
            break;

        case 8:
            path += "8";
            break;
        }
        path += ".ogg";
    }

    for(unsigned int i(0); i < m_Sound.size(); i++)
    {
        if(m_Sound[i]->getBuffer() == &m_SoundBuffer.get(path))
        {
            m_Sound[i]->play();
            return;
        }
    }

    m_Sound.push_back(new sf::Sound(m_SoundBuffer.get(path)));
    m_Sound.back()->play();
}

void RessourceManager::playBGM(const std::string name)
{
    if(!DEBUG_MODE)
    {
        if(!("data\\sounds\\" + name == m_pathBGM))
        {
            m_pathBGM = "data\\sounds\\" + name;
            m_BGM.openFromFile(m_pathBGM);
            m_BGM.play();
        }else
            m_BGM.play();
    }
}

void RessourceManager::pauseBGM()
{
    if(m_BGM.getStatus() == sf::Music::Playing)
        m_BGM.pause();
}

void RessourceManager::resumeBGM()
{
    if(m_BGM.getStatus() == sf::Music::Paused)
        m_BGM.play();
}

void RessourceManager::stopBGM()
{
    if(m_BGM.getStatus() == sf::Music::Playing)
        m_BGM.stop();
}

void RessourceManager::drawCursor()
{
    m_Window->draw(*m_hud_curseur.top());
}

void RessourceManager::setMousePosition(const sf::Vector2i& pos)
{
    m_hud_curseur.top()->setPosition(pos.x - m_hud_curseur.top()->getTextureRect().width / 2,
                                     pos.y - m_hud_curseur.top()->getTextureRect().height / 2);
}

sf::Vector2i RessourceManager::getMousePosition()
{
    return sf::Vector2i(m_hud_curseur.top()->getPosition().x + m_hud_curseur.top()->getTextureRect().width / 2,
                        m_hud_curseur.top()->getPosition().y + m_hud_curseur.top()->getTextureRect().height / 2);
}

void RessourceManager::setMouseCursor(const std::string& name)
{
    std::string path("data\\graphics\\window\\hud\\" + name + ".png");
    sf::Vector2i pos (0, 0);

    if(m_hud_curseur.size() > 0)
        pos = getMousePosition();

    m_hud_curseur.push(new sf::Sprite(getTexture(path)));

    setMousePosition(pos);
}

void RessourceManager::deleteMouseCursor()
{
    if(m_hud_curseur.size() > 1)
    {
        killTexture(m_hud_curseur.top()->getTexture());
        delete m_hud_curseur.top();
        m_hud_curseur.pop();
    }
}
