#ifndef RESSOURCEMANAGER_HPP
#define RESSOURCEMANAGER_HPP

// C++
#include <vector>
#include <stack>
// SFML
#include <SFML/Graphics.hpp>
#include <SFML/Audio.hpp>
// Projet
#include "manager_texture.hpp"
#include "manager_font.hpp"
#include "manager_soundbuffer.hpp"

class Application;
class Map;

class RessourceManager
{
public:
    RessourceManager();
    ~RessourceManager();

    void setApplication(Application*);
    Application& getApplication();

    Map* getMap();
    void setMap(Map*);

    void setWindow(sf::RenderWindow*);
    sf::RenderWindow& getWindow();

    const sf::Texture& getTexture(const std::string&);
    void killTexture(const std::string&);
    void killTexture(const sf::Texture*);

    const sf::Font& getFont(const std::string&);

    void playSound(std::string);
    void playBGM(const std::string);
    void pauseBGM();
    void resumeBGM();
    void stopBGM();

    void drawCursor();
    void setMousePosition(const sf::Vector2i&);
    sf::Vector2i getMousePosition();
    void setMouseCursor(const std::string&);
    void deleteMouseCursor();

private:
    // C++
    std::string m_pathBGM;
    // SFML
    sf::RenderWindow* m_Window;
    std::vector<sf::Sound*> m_Sound;
    sf::Music m_BGM;
    std::stack<sf::Sprite*> m_hud_curseur;
    // Projet
    Application* m_Application;
    Map* m_Map;
    Manager_Texture m_Textures;
    Manager_Font m_Fonts;
    Manager_SoundBuffer m_SoundBuffer;
};

#endif // RESSOURCEMANAGER_HPP
