#include "screen.hpp"
#include "application.hpp"
#include "collisions.hpp"
#include "map.hpp"

MapInterlude::MapInterlude(RessourceManager* rm, std::string lvlName) : GameState(rm, true, false),
    m_nextLevelName(lvlName), m_tempsBloque(0.5), m_peuxEtrePasse(false)
{
    m_background = new Sprite(m_RManager->getTexture("data\\graphics\\window\\mapInterlude.png"));
}

MapInterlude::~MapInterlude()
{
    m_RManager->killTexture(m_background->getTexture());
}

void MapInterlude::draw()
{
    m_RManager->getWindow().draw(*m_background);
}

bool MapInterlude::event(const sf::Event& e)
{
    if(e.type == sf::Event::KeyPressed && m_peuxEtrePasse)
    {
        m_RManager->getMap()->changeMap(m_nextLevelName);
        m_RManager->getApplication().popGameState();
    }

    return m_modal;
}

void MapInterlude::update(float t)
{
    if(!m_peuxEtrePasse)
    {
        m_tempsBloque -= t;

        if(m_tempsBloque <= 0)
            m_peuxEtrePasse = true;
    }

}

GameOver::GameOver(RessourceManager* rm) : GameState(rm, true, false)
{
    m_background = new Sprite(m_RManager->getTexture("data\\graphics\\window\\gameOver\\gameOver.png"));

    m_recommencer.setTexture(m_RManager->getTexture("data\\graphics\\window\\gameOver\\recommencer.png"));
    m_recommencer.setPosition(250, 300);

    m_retourMenu.setTexture(m_RManager->getTexture("data\\graphics\\window\\gameOver\\menuPrincipal.png"));
    m_retourMenu.setPosition(250, 400);

    m_levelName = m_RManager->getMap()->getMapName();

    m_RManager->playSound("data\\sounds\\gameOver.ogg");
}

GameOver::~GameOver() {}

void GameOver::draw()
{
    m_RManager->getWindow().draw(*m_background);
    m_RManager->getWindow().draw(m_recommencer);
    m_RManager->getWindow().draw(m_retourMenu);
}

bool GameOver::event(const sf::Event& e)
{
    if(e.type == sf::Event::MouseButtonReleased)
    {
        if(collisionSourisSprite(e.mouseButton.x, e.mouseButton.y, m_recommencer))
        {
            m_RManager->getMap()->changeMap(m_levelName);
            m_RManager->getMap()->getPlayer()->setVie(-1);
            m_RManager->getApplication().popGameState();
        }else if(collisionSourisSprite(e.mouseButton.x, e.mouseButton.y, m_retourMenu))
        {
            m_RManager->getApplication().mainMenu();
        }
    }
    return m_modal;
}

ScreenCommandes::ScreenCommandes(RessourceManager* rm) : GameState(rm, true, true), m_idScreen(0)
{
    loadScreen();
}

ScreenCommandes::~ScreenCommandes()
{
    m_RManager->killTexture(m_screen.getTexture());
}

void ScreenCommandes::draw()
{
    m_RManager->getWindow().draw(m_screen);
}

bool ScreenCommandes::event(const sf::Event& e)
{
    if(e.type == sf::Event::KeyReleased)
        loadScreen();
}

void ScreenCommandes::loadScreen()
{
    m_RManager->killTexture(m_screen.getTexture());

    switch(m_idScreen)
    {
    case 0:
        m_screen.setTexture(m_RManager->getTexture("data\\graphics\\window\\commandes\\01.png"));
        m_idScreen++;
        break;

    default:
        m_RManager->getApplication().popGameState();
        break;
    }
}
