#include "scriptedentity.hpp"
#include "player.hpp"
#include "map.hpp"
#include "application.hpp"
#include "dialogue.hpp"
#include "consoleminigame.hpp"

ScriptedEntity::ScriptedEntity(RessourceManager* rm, Sprite* sp, const std::string path, int x, int y) :
    Entity(rm, sp, SpriteFilled, x, y), m_path(path), m_typeDeclenchement(Dont), m_seRepete(false)
{
    load();
}

ScriptedEntity::~ScriptedEntity() {}

void ScriptedEntity::update(float t)
{
    Entity::update(t);

    if(m_typeDeclenchement == AfterWait)
    {
        m_tempsAvantDeclenchement -= t;
        if(m_tempsAvantDeclenchement <= 0)
            activate();
    }
}

void ScriptedEntity::onShoot()
{
    if(m_typeDeclenchement == OnShoot)
        activate();
}

void ScriptedEntity::onCollision()
{
    if(m_typeDeclenchement == OnCollision)
        activate();
}

void ScriptedEntity::activate()
{
    std::ifstream fichier(m_path.c_str());
    std::string op;

    std::cout << " -- Activation du script : " << m_path << std::endl;

    while(!fichier.eof())
    {
        fichier >> op;

        if(op == "[SetPlayerPosition]")
        {
            int x, y;
            fichier >> x >> y;

            m_RManager->getMap()->getPlayer()->setPosition(x, y);
        }
        else if(op == "|")
        {
            getline(fichier, op);
        }
        else if(op == "[ShowDialogue]")
        {
            int idAActiver;

            fichier >> op >> idAActiver;

            m_RManager->getApplication().addGameState(new Dialogue(m_RManager, op, idAActiver));
        }
        else if(op == "[SetSolid]")
        {
            fichier >> m_solid;
        }
        else if(op == "[PlayerCanShoot]")
        {
            bool peux;
            fichier >> peux;

            m_RManager->getMap()->getPlayer()->peuxTirer(peux);
        }else if(op == "[PlusOuMoins]")
        {
            int nbChiffre;

            fichier >> nbChiffre;

            m_RManager->getApplication().addGameState(new PlusOuMoins(m_RManager, nbChiffre));
        }
        else if(op == "[ActivateEntity]")
        {
            int id;
            fichier >> id;

            m_RManager->getMap()->activerEntite(id);
        }
        else if(op == "[MasterMindCode]")
        {
            int nbChiffre, idAActiver;

            fichier >> nbChiffre >> idAActiver;

            m_RManager->getApplication().addGameState(new MasterMind(m_RManager, nbChiffre, idAActiver));
        }
        else if(op == "[SetEntityVisible]")
        {
            int id;
            bool visible;

            fichier >> id >> visible;

            m_RManager->getMap()->setEntityVisible(id, visible);
        }
        else if(op == "[PlayerCanToggleGravity]")
        {
            bool peux;

            fichier >> peux;

            m_RManager->getMap()->getPlayer()->canToggleGravity(peux);
        }
        else if(op == "[SetSolid]")
        {
            fichier >> m_solid;
        }
    }

    if(m_seRepete)
        load();
    else
        m_typeDeclenchement = Dont;

    m_sprite->setEtatY(1);

    fichier.close();
}

void ScriptedEntity::load()
{
    std::ifstream fichier(m_path.c_str());

    if(!fichier.is_open())
        std::cout << " *** ERROR : ERREUR CHARGEMENT SCRIPT\n    * " << m_path << std::endl;

    std::string op;

    while(!fichier.eof())
    {
        fichier >> op;

        if(op == "[TypeDeclenchement]")
        {
            fichier >> op;

            if(op == "OnMapLoading")
            {
                m_typeDeclenchement = AfterWait;
                m_tempsAvantDeclenchement = 0;
            }
            else if(op == "OnCollision")
            {
                m_typeDeclenchement = OnCollision;
            }
            else if (op == "OnShoot")
            {
                m_typeDeclenchement = OnShoot;
            }
            else if (op == "AfterWait")
            {
                m_typeDeclenchement = AfterWait;

                fichier >> m_tempsAvantDeclenchement;
            }
        }
        else if(op == "|")
        {
            getline(fichier, op);
        }
        else if(op == "[IsSolid]")
        {
            fichier >> m_solid;
        }
    }

    fichier.close();
}
