#include "sprite.hpp"

Sprite::Sprite(const sf::Texture& text)
{
    sf::Sprite::setTexture(text);
}

Sprite::~Sprite() {}

bool Sprite::update(float t) { return false;}

void Sprite::setEtatY(int y) {}

void Sprite::setEtatX(int x) {}

RepeatedSprite::RepeatedSprite(const sf::Texture& text, float vBas, float vGauche) : Sprite(text),
    m_vBas(vBas), m_vGauche(vGauche)
{}

RepeatedSprite::~RepeatedSprite() {}

bool RepeatedSprite::update(float t)
{
    setTextureRect(sf::IntRect(getTextureRect().left + m_vGauche * t, getTextureRect().top - m_vBas * t,
                               getTextureRect().width, getTextureRect().height));

    return false;
}

AnimatedSprite::AnimatedSprite(const sf::Texture& text, int nbEtatX, int nbEtatY, float tpsEntreImages, bool changeEtatYToutSeul) :
    Sprite(text), m_nbEtatX(nbEtatX), m_nbEtatY(nbEtatY), m_tpsEntreImages(tpsEntreImages), m_tpsDepuisImage(m_tpsEntreImages),
    m_etatX(0), m_etatY(0), m_changeEtatYToutSeul(changeEtatYToutSeul)
{
    m_largeurSprite = getTextureRect().width / m_nbEtatX;
    m_hauteurSprite = getTextureRect().height / m_nbEtatY;

    setTextureRect(sf::IntRect(m_etatX * m_largeurSprite, m_etatY * m_hauteurSprite,
                               m_largeurSprite, m_hauteurSprite));

}

AnimatedSprite::~AnimatedSprite() {}

bool AnimatedSprite::update(float t)
{
    if(m_tpsEntreImages != -1)
    {
        m_tpsDepuisImage += t;

        if(m_tpsDepuisImage > m_tpsEntreImages)
        {
            if(m_etatX < m_nbEtatX - 1)
            {
                m_etatX++;
            }else if(m_changeEtatYToutSeul)
            {
                if(m_etatY < m_nbEtatY - 1)
                {
                    m_etatY++;
                }else
                {
                    m_etatY = 0;
                }

                m_etatX = 0;
            }else if(!m_changeEtatYToutSeul)
            {
                m_etatX = 0;
            }

            setTextureRect(sf::IntRect(m_etatX * m_largeurSprite, m_etatY * m_hauteurSprite,
                                       m_largeurSprite, m_hauteurSprite));

            m_tpsDepuisImage = 0.f;

            return true;
        }
    }
}

void AnimatedSprite::setEtatY(int etatY)
{
    if(etatY < m_nbEtatY && etatY >= 0)
    {
        m_etatY = etatY;

        setTextureRect(sf::IntRect(m_etatX * m_largeurSprite, m_etatY * m_hauteurSprite,
                               m_largeurSprite, m_hauteurSprite));
    }
}

void AnimatedSprite::setEtatX(int etatX)
{
    if(etatX < m_nbEtatX && etatX >= 0)
    {
        m_etatX = etatX;

    setTextureRect(sf::IntRect(m_etatX * m_largeurSprite, m_etatY * m_hauteurSprite,
                               m_largeurSprite, m_hauteurSprite));
    }
}

int AnimatedSprite::getEtatX()
{
    return m_etatX;
}

int AnimatedSprite::getEtatY()
{
    return m_etatY;
}
