#include "trigger.hpp"

ShootableTrigger::ShootableTrigger(RessourceManager* rm, Sprite* sprite, Entity* destination, int x, int y, float delay) :
    Entity(rm, sprite, SpriteFilled, x, y, true, false, true, false, delay), m_destination(destination)
{

}

ShootableTrigger::~ShootableTrigger() {}

void ShootableTrigger::onShoot()
{
    activate();
}

void ShootableTrigger::activate()
{
    if(!m_active)
    {
        m_destination->activate();
        m_active = true;
        m_sprite->setEtatY(1);
        m_RManager->playSound("data\\sounds\\trigger.ogg");
    }
}

void ShootableTrigger::update(float t)
{
    if(m_active && m_delay != -1)
    {
        m_tpsDepuisActivation += t;

        if(m_tpsDepuisActivation >= m_delay)
        {
            m_active = false;
            m_tpsDepuisActivation = 0.f;
            m_sprite->setEtatY(0);
        }
    }

    if(m_sprite->update(t))
        refreshHitbox();
}

ActivableTrigger::ActivableTrigger(RessourceManager* rm, Sprite* sprite, Entity* destination, int x, int y, float delay) :
    Entity(rm, sprite, SpriteFilled, x, y, false, false, false, true, delay), m_destination(destination)
{}

ActivableTrigger::~ActivableTrigger() {}

void ActivableTrigger::activate()
{
    if(!m_active)
    {
        m_destination->activate();
        m_active = true;
        m_sprite->setEtatY(1);
        m_RManager->playSound("data\\sounds\\trigger.ogg");
    }
}

void ActivableTrigger::update(float t)
{
    if(m_active && m_delay != -1)
    {
        m_tpsDepuisActivation += t;

        if(m_tpsDepuisActivation >= m_delay)
        {
            m_active = false;
            m_tpsDepuisActivation = 0.f;
            m_sprite->setEtatY(0);
        }
    }

    if(m_sprite->update(t))
        refreshHitbox();
}

TimerActivableTrigger::TimerActivableTrigger(RessourceManager* rm, Sprite* sp, Entity* dest, int x, int y, float delay) :
    ActivableTrigger(rm, sp, dest, x, y, delay) {}

TimerActivableTrigger::~TimerActivableTrigger() {}

void TimerActivableTrigger::update(float t)
{
    bool actif = m_active;

    ActivableTrigger::update(t);

    if(actif != m_active)
    {
        m_RManager->playSound("data\\sounds\\trigger.ogg");
        m_destination->activate();
    }
}

TimerShootableTrigger::TimerShootableTrigger(RessourceManager* rm, Sprite* sp, Entity* dest, int x, int y, float delay) :
    ShootableTrigger(rm, sp, dest, x, y, delay) {}

TimerShootableTrigger::~TimerShootableTrigger() {}

void TimerShootableTrigger::update(float t)
{
    bool actif = m_active;

    ShootableTrigger::update(t);

    if(actif != m_active)
    {
        m_RManager->playSound("data\\sounds\\trigger.ogg");
        m_destination->activate();
    }
}
